using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.ServiceModel;
using System.Windows.Forms;

namespace Client
{
	public partial class ResourceForm : Form
	{
		private delegate void ShowMessageDelegate(string message);

		private enum Resource
		{
			Coal = 0,
			Diamond = 1
		}

		public ResourceForm()
		{
			InitializeComponent();
		}

		private void AccessResource(object state)
		{
			string message = null;
			IResourceAccessContract proxy = null;
			try
			{

				Resource resource = (Resource)state;
				proxy = new ChannelFactory<IResourceAccessContract>("ResourceAccessService").CreateChannel();
				switch (resource)
				{
					case Resource.Coal:
						message = proxy.AccessCoal();
						break;
					case Resource.Diamond:
						message = proxy.AccessDiamond();
						break;
				}


			}
			catch (Exception exception)
			{
				message = exception.Message;
			}
			finally
			{
				if (proxy != null)
				{
					((IChannel)proxy).Close();
				}
			}

			this.Invoke(new ShowMessageDelegate(this.ShowMessage), new object[] { message });
		}



		private void ShowMessage(string message)
		{
			MessageBox.Show(message);
		}

		private void CoalButton_Click(object sender, EventArgs e)
		{
			ThreadPool.QueueUserWorkItem(new WaitCallback(this.AccessResource), Resource.Coal);
		}


		private void DiamondButton_Click(object sender, EventArgs e)
		{
			ThreadPool.QueueUserWorkItem(new WaitCallback(this.AccessResource), Resource.Diamond);
		}
	}
}