﻿

using System;
using System.Configuration;
using System.Messaging;
using System.ServiceModel;

namespace WCFHandsOn
{
    // Define a service contract. 
    [ServiceContract]
    public interface IBillPayment
    {
        [OperationContract(IsOneWay=true)]
        void PayBill(int AccountNbr, int ElectricAccountNbr, double Amount);
        
    }

    // Service class which implements the service contract.
    // Added code to write output to the console window
    public class BillPaymentService : IBillPayment
    {
        [OperationBehavior]
        public void PayBill(int AccountNbr, int ElectricAccountNbr, double Amount)
        {
            
            Console.WriteLine("Received Request to Pay Electric Company Account {0}  the amount of {1} from local account {2}", ElectricAccountNbr, Amount.ToString(), AccountNbr);
        }



        // Host the service within this EXE console application.
        public static void Main()
        {
            // Get MSMQ queue name from app settings in configuration
            string queueName = ConfigurationManager.AppSettings["queueName"];

            // Create the transacted MSMQ queue if necessary.
            if (!MessageQueue.Exists(queueName))
                MessageQueue.Create(queueName, true);

            // Get the base address that is used to listen for WS-MetaDataExchange requests
            // This is useful to generate a proxy for the client
            string baseAddress = ConfigurationManager.AppSettings["baseAddress"];

            // Create a ServiceHost for the BillPayment type.
            using (ServiceHost serviceHost = new ServiceHost(typeof(BillPaymentService), new Uri(baseAddress)))
            {
                
                    serviceHost.Open();

                
                    Console.WriteLine("The Bill Payment Service is online.");
                    Console.WriteLine("Press <ENTER> to terminate service.");
                    Console.WriteLine();
                    Console.ReadLine();

                    // Close the ServiceHostBase to shutdown the service.
                    serviceHost.Close();
            }
        }

    }

}
