using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.Transactions;

namespace ConsumerBankingService
{
    [ServiceContract(Session=true)]
    public interface IConsumerBankingCore
    {
        
        [OperationContract]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        bool Deposit(int accountNumber, decimal amount);
        
        [OperationContract]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        bool Withdraw(int accountNumber, decimal amount);
        
        [OperationContract]
        decimal GetBalance(int accountNumber);
    }

    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Shareable)]
    class ConsumerBankingCore : IConsumerBankingCore
    {
        //NOTE: A balance of $100 is coded into the class to avoid
        //the requirement for the user to have a database installed.
        private decimal _balance = 100;
        public bool Deposit(int accountNumber, decimal amount)
        {
            
               if (amount > 0)
               {
                   _balance = _balance + amount;
                   Console.WriteLine("Depositing {0} into Account {1}", amount, accountNumber);
                   return true;
               }
               else
               {
                   return false;
               }
        }
        public bool Withdraw(int accountNumber, decimal amount)
        {
           
            if (amount <= _balance)
            {
                _balance = _balance - amount;
                Console.WriteLine("Withdrawing {0} from Account {1}", amount, accountNumber);
                return true;
            }
            else
            {
                return false;
            }
        }
        public decimal GetBalance(int accountNumber)
        { return _balance; }
    
    }
}
