using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace PublicationSubscription
{
	public class BufferedSubscriptionManager: IBufferedSubscriptionManager
	{
		private Queue<IBufferedDataPointValue> dataPointValueBuffer = null;
		private ISubscriptionManager subscriptionManager = null;
		private object dataPointValueBufferLock = null;
		private int bufferCapacity;

		public event DataAvailableHandler DataAvailable = null;
		
		private BufferedSubscriptionManager()
		{
			this.dataPointValueBuffer = new Queue<IBufferedDataPointValue>();
			this.dataPointValueBufferLock = new object();
		}

		public BufferedSubscriptionManager(string subscriber, int bufferCapacity):this()
		{
			this.subscriptionManager = new SubscriptionManager(subscriber, this.NextValueHandler);
			this.bufferCapacity = bufferCapacity;
		}

		private void NextValueHandler(IDataPoint sender, byte[] newValue)
		{
			lock (this.dataPointValueBufferLock)
			{
				this.dataPointValueBuffer.Enqueue(new BufferedDataPointValue(sender.Identifier, newValue));
				while (this.dataPointValueBuffer.Count > this.bufferCapacity)
				{
					this.dataPointValueBuffer.Dequeue();
				}
			}

			if (this.DataAvailable != null)
			{
				this.DataAvailable(this);
			}
		}

		#region IDataSubscriptionManager Members

		string ISubscriptionManager.Subscriber
		{
			get 
			{ 
				return this.subscriptionManager.Subscriber; 
			}
		}

		IDataPoint[] ISubscriptionManager.Subscriptions
		{
			get 
			{ 
				return this.subscriptionManager.Subscriptions; 
			}
		}

		SubscriptionManagerState ISubscriptionManager.State
		{
			get 
			{
				return this.subscriptionManager.State;
			}
		}

		void ISubscriptionManager.AddSubscription(IDataPoint dataPoint)
		{
			this.subscriptionManager.AddSubscription(dataPoint);
		}


		

		#endregion

		#region IDisposable Members

		void IDisposable.Dispose()
		{
			lock (this.dataPointValueBufferLock)
			{
				this.dataPointValueBuffer = null;
			}
			this.subscriptionManager.Dispose();
		}

		#endregion

		#region IBufferedDataSubscriptionManager Members

		Queue<IBufferedDataPointValue> IBufferedSubscriptionManager.DataPointValueBuffer
		{
			get 
			{
				lock (this.dataPointValueBufferLock)
				{
					return this.dataPointValueBuffer;
				}
			}
		}

		int IBufferedSubscriptionManager.BufferCapacity
		{
			get
			{
				return this.bufferCapacity;
			}
		}

		

		#endregion
}
}
