using System;
using System.Collections.Generic;
using System.Configuration;
using System.ServiceModel;
using System.Text;

namespace PublicationSubscription
{
	public class Subscriber: ISubscriber, IDisposable
	{
		IPublisher publisher = null;

		public Subscriber()
		{
			publisher = new ChannelFactory<IPublisher>("SubscriptionService").CreateDuplexChannel(new InstanceContext(this));
		}

		public void Subscribe()
		{
			KnownDataPoint[] dataPoints = this.publisher.GetKnownDataPoints();
			bool subscriptionAccepted = false;
			this.publisher.Subscribe(dataPoints, out subscriptionAccepted);
			Console.WriteLine(subscriptionAccepted ? "Subscription accepted." : "Subscription rejected.");
			
		}

		#region ISubscriber Members

		void ISubscriber.Notify(Guid dataPointIdentifier, byte[] value)
		{
			Console.WriteLine("Notified of value {0} of data point {1}.", BitConverter.ToInt32(value,0), dataPointIdentifier.ToString());
		}

		

		#endregion

		#region IDisposable Members

		void IDisposable.Dispose()
		{
			try
			{
				if (this.publisher != null)
				{
					IChannel publisherChannel = (IChannel)publisher;
					if (publisherChannel.State == CommunicationState.Opened)
					{
						publisherChannel.Close();
					}
				}
			}
			catch (Exception)
			{
			}

		}

		#endregion
}
}
