
//  Copyright (c) Microsoft Corporation.  All Rights Reserved.

using System;
using System.Text;

namespace Microsoft.ServiceModel.Samples
{
    // Define the Purchase Order Line Item
    [Serializable]
    public class PurchaseOrderLineItem
    {
        
        public string productIdentifier;
        public float unitCost;
        public int quantity;

        public override string ToString()
        {
            String displayString = "Order LineItem: " + quantity + " of " + productIdentifier + " @unit price: $" + unitCost + "\n";
            return displayString;
        }

        public float TotalCost
        {
            get { return unitCost * quantity; }
        }
    }
    public enum OrderStates
    {
        Pending,
        Processed,
        Shipped
    }
    // Define Purchase Order
    [Serializable]
    public class PurchaseOrder
    {
        public string orderIdentifier;
        public string customerIdentifier;
        public PurchaseOrderLineItem[] orderLineItems;
        private OrderStates orderStatus;

        public float TotalCost
        {
            get
            {
                float totalCost = 0;
                foreach (PurchaseOrderLineItem lineItem in orderLineItems)
                    totalCost += lineItem.TotalCost;
                return totalCost;
            }
        }

        public OrderStates Status
        {
            get
            {
                return orderStatus;
            }
            set
            {
                orderStatus = value;
            }
        }

        public override string ToString()
        {
            StringBuilder buffer = new StringBuilder("Purchase Order: " + orderIdentifier + "\n");
            buffer.Append("\tCustomer: " + customerIdentifier + "\n");
            buffer.Append("\tOrderDetails\n");

            foreach (PurchaseOrderLineItem lineItem in orderLineItems)
            {
                buffer.Append("\t\t" + lineItem.ToString());
            }

            buffer.Append("\tTotal cost of this order: $" + TotalCost + "\n");
            buffer.Append("\tOrder status: " + Status + "\n");
            return buffer.ToString();
        }
    }

}