﻿
//  Copyright (c) Microsoft Corporation.  All Rights Reserved.

using System;
using System.Collections.Generic;
using System.Text;
using System.Messaging;
using System.Configuration;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace Microsoft.ServiceModel.Samples
{
    [ServiceContract(Namespace = "http://Microsoft.ServiceModel.Samples")]
    [KnownType(typeof(PurchaseOrder))]
    public interface IOrderSubscriber
    {
        [OperationContract(IsOneWay = true, Action = "*")]
        void Notify(MsmqMessage<PurchaseOrder> message);
    }

	public class Publisher
	{
		public static void Main(string[] args)
		{
            Console.WriteLine("Press any key to proceed.");
            Console.ReadKey();

            PurchaseOrder order = new PurchaseOrder();
            order.customerIdentifier = "somecustomer.com";
            order.orderIdentifier = Guid.NewGuid().ToString();

            PurchaseOrderLineItem firstLineItem = new PurchaseOrderLineItem();
            firstLineItem.productIdentifier = "Blue Widget";
            firstLineItem.quantity = 54;
            firstLineItem.unitCost = 29.99F;

            PurchaseOrderLineItem secondLineItem = new PurchaseOrderLineItem();
            secondLineItem.productIdentifier = "Red Widget";
            secondLineItem.quantity = 890;
            secondLineItem.unitCost = 45.89F;

            order.orderLineItems = new PurchaseOrderLineItem[] { firstLineItem, secondLineItem };

            IOrderSubscriber proxy = new ChannelFactory<IOrderSubscriber>("OrderPullPoint").CreateChannel();

            proxy.Notify( new MsmqMessage<PurchaseOrder>(order));
            ((IChannel)proxy).Close();

            
            Console.WriteLine("Placed the order:{0}", order);
            Console.WriteLine("Press any key to terminate the client.");
            Console.ReadKey();
		}
	}
}
