﻿
//  Copyright (c) Microsoft Corporation.  All Rights Reserved.

using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Runtime.Serialization;
using System.Configuration;
using System.Messaging;

namespace Microsoft.ServiceModel.Samples
{
    [ServiceContract(Namespace = "http://Microsoft.ServiceModel.Samples")]
    [KnownType(typeof(PurchaseOrder))]
    public interface IOrderSubscriber
    {
        [OperationContract(IsOneWay = true, Action = "*")]
        void Notify(MsmqMessage<PurchaseOrder> message);
    }

    public class OrderSubscriber : IOrderSubscriber
    {
        public void Notify(MsmqMessage<PurchaseOrder> message)
        {
            PurchaseOrder order = (PurchaseOrder)message.Body;
            Random statusIndexer = new Random();
            order.Status = (OrderStates)statusIndexer.Next(3);
            Console.WriteLine("Processing {0} ", order);
        }

        public static void Main()
        {
            string queueName = ConfigurationManager.AppSettings["orderQueueName"];

            if (!(MessageQueue.Exists(queueName)))
            {
                MessageQueue.Create(queueName);
                MessageQueue queue = new MessageQueue(queueName);
                queue.MulticastAddress = 
                    ConfigurationManager.AppSettings["multicastAddress"];
            }

            using (ServiceHost serviceHost = new ServiceHost(typeof(OrderSubscriber)))
            {
                serviceHost.Open();

                Console.WriteLine("The service is ready.");
                Console.WriteLine("Press any key to terminate the service.");
                Console.ReadLine();

                serviceHost.Close();
            }
        }
        
    }
}

