using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

namespace PublicationSubscription
{
	public class SubscriptionStreamControlSignal: ISubscriptionStreamControlSignal
	{
		private Guid dataPointIdentifier;
		private long dataPointValueSize;
		
		protected SubscriptionStreamControlSignal()
		{
		}

		public SubscriptionStreamControlSignal(Guid dataPointIdentifier, long valueSize)
		{
			this.dataPointIdentifier = dataPointIdentifier;
			this.dataPointValueSize = valueSize;
		}

		public SubscriptionStreamControlSignal(byte[] controlSignalBuffer)
		{
			byte[] segmentBuffer = new byte[this.SizeOfDataPointIdentifier];
			Array.Copy(controlSignalBuffer, 0, segmentBuffer, 0, segmentBuffer.Length);

			this.dataPointIdentifier = new Guid(segmentBuffer);
			
			this.dataPointValueSize = BitConverter.ToInt64(controlSignalBuffer, this.SizeOfDataPointIdentifier);
		}

		public static byte Size
		{
			get
			{
				return (byte)(Marshal.SizeOf(typeof(Guid)) + Marshal.SizeOf(typeof(long)));
			}
		}

		private int SizeOfDataPointIdentifier
		{
			get
			{
				return Marshal.SizeOf(this.dataPointIdentifier.GetType());
			}
		}

		private int SizeofDataPointValueSize
		{
			get
			{
				return Marshal.SizeOf(this.dataPointValueSize.GetType());
			}
		}

		#region IDataSubscriptionStreamControlSignal Members

		Guid ISubscriptionStreamControlSignal.DataPointIdentifier
		{
			get
			{
				return this.dataPointIdentifier;
			}
		}

		long ISubscriptionStreamControlSignal.DataPointValueSize
		{
			get
			{
				return this.dataPointValueSize;
			}
		}

		byte[] ISubscriptionStreamControlSignal.ToByteArray()
		{
			byte[] controlSignalBuffer = new byte[this.SizeOfDataPointIdentifier + this.SizeofDataPointValueSize];
						
			Array.Copy(this.dataPointIdentifier.ToByteArray(), 0, controlSignalBuffer, 0, this.SizeOfDataPointIdentifier);
			
			Array.Copy(BitConverter.GetBytes(this.dataPointValueSize), 0, controlSignalBuffer, this.SizeOfDataPointIdentifier, this.SizeofDataPointValueSize);
			
			return controlSignalBuffer;
		}

		#endregion
}
}
