using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Text;

namespace PublicationSubscription
{
	public class Subscriber: ISubscriber
	{
		public Subscriber()
		{
		}

		public void Subscribe()
		{
			using (PublisherProxy proxy = new PublisherProxy("SubscriptionService"))
			{
				IPublisher publisher = (IPublisher)proxy;
				KnownDataPoint[] dataPoints = publisher.GetKnownDataPoints();

				bool subscriptionAccepted = false;
				publisher.Subscribe(dataPoints, out subscriptionAccepted);
				Console.WriteLine(subscriptionAccepted ? "Subscription accepted." : "Subscription rejected.");
				proxy.Close();
			}
		}

		private void ReadValueHandler(object sender, string dataPointIdentifier, string value)
		{
			Console.WriteLine("Read value {0} of data point {1}.", value, dataPointIdentifier);
		}

		private void EndOfStreamHandler(object sender)
		{
			Console.WriteLine("End of stream.");
		}

		#region ISubscriber Members

		void ISubscriber.Notify(Stream stream)
		{
			Console.WriteLine("Receiving a stream.");

			INotificationStreamReader reader = new NotificationStreamReader(stream);
			reader.EndOfStream += new EndOfStreamHandler(this.EndOfStreamHandler);
			reader.NextValue += new ReadValueHandler(this.ReadValueHandler);
			reader.Start();

			Console.WriteLine("Finished reading a stream.");
		}

		

		#endregion
}
}
