using System;
using System.Collections.Generic;
using System.Configuration;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Security.Cryptography;
using System.Security.Cryptography.Xml;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Security.Tokens;
using System.Text;


namespace WindowsCommunicationFoundationHandsOn.School
{
	public class SecurityValidator : X509CertificateValidator
	{
		public override void Validate(X509Certificate2 certificate)
		{
		}
	}

	public class QuizChild: IQuizQuestion, IQuizResponse, IQuizManagement, IDisposable
	{
		private const string BaseAddressKey = "baseAddress";
		private const string MeshIdentifierKey = @"meshIdentifier";
		private const string ResponseMeshIdentifierKey = @"responseMeshIdentifier";
		private const string QuizQuestionEndpointKey = @"quizQuestionEndpoint";
		private const string QuizResponseEndpointKey = @"quizResponseEndpoint";
		private const string QuizManagementEndpointKey = @"quizManagementEndpoint";
		private const string PeerChannelAddressPrefix = @"net.p2p://";
		private const string CurrentUserCertificateStore = @"My";
		private const string CertificateSubjectName = @"Woodgrove";

		private ServiceHost resolverService = null;
		private IQuizQuestionChannel questionProxy = null;
		private IQuizManagementChannel managementProxy = null;
		private IQuizResponseChannel responseProxy = null;
		private PeerSecurityBehavior peerSecurity = null;
		
		private MainForm form = null;
		private Student student = new Student();

		public QuizChild(MainForm form, string[] parameters)
		{
			this.form = form;
						
			this.student.Name = parameters[0];
			int port = int.Parse(parameters[1]);
			
			InstanceContext site = new InstanceContext(this);

			string endpoint = null;
			EndpointAddress address = null;
			string meshIdentifier = ConfigurationManager.AppSettings[QuizChild.MeshIdentifierKey];

			//Management
			endpoint = ConfigurationManager.AppSettings[QuizChild.QuizManagementEndpointKey];
			address = new EndpointAddress(string.Concat(QuizChild.PeerChannelAddressPrefix, meshIdentifier, "/", endpoint));
			ChannelFactory<IQuizManagementChannel> managementChannelFactory = new ChannelFactory<IQuizManagementChannel>(site, this.BuildBinding(port), address);
			managementChannelFactory.Open();
			managementProxy = (IQuizManagementChannel)managementChannelFactory.CreateDuplexChannel();
			((IChannel)managementProxy).Open();

			//Question
			endpoint = ConfigurationManager.AppSettings[QuizChild.QuizQuestionEndpointKey];
			address = new EndpointAddress(string.Concat(QuizChild.PeerChannelAddressPrefix, meshIdentifier, "/", endpoint));
			ChannelFactory<IQuizQuestionChannel> questionChannelFactory = new ChannelFactory<IQuizQuestionChannel>(site, this.BuildBinding(port), address);
			questionChannelFactory.Open();
			questionProxy = (IQuizQuestionChannel)questionChannelFactory.CreateDuplexChannel();
			((IChannel)questionProxy).Open();

			//Response
			endpoint = ConfigurationManager.AppSettings[QuizChild.QuizResponseEndpointKey];
			address = new EndpointAddress(string.Concat(QuizChild.PeerChannelAddressPrefix, meshIdentifier, "/", endpoint, "_", this.student.Name));
			ChannelFactory<IQuizResponseChannel> responseChannelFactory = new ChannelFactory<IQuizResponseChannel>(site, this.BuildBinding(port), address);
			responseChannelFactory.Open();
			responseProxy = (IQuizResponseChannel)responseChannelFactory.CreateDuplexChannel();
			((IChannel)responseProxy).Open();

			//Announce student to teacher:
			this.Join();

			
			
			
		}

		private X509Certificate2 GetCertificate()
		{
			X509Store store = new X509Store(QuizChild.CurrentUserCertificateStore, StoreLocation.LocalMachine);
			store.Open(OpenFlags.ReadOnly);
			X509CertificateCollection certificateCollection = store.Certificates.Find(X509FindType.FindBySubjectName, QuizChild.CertificateSubjectName, false);
			return (X509Certificate2)certificateCollection[0];
		}

		private NetPeerTcpBinding BuildBinding(int port)
		{
			NetPeerTcpBinding binding = new NetPeerTcpBinding();
			binding.MaxMessageSize = int.MaxValue;
			binding.PeerNodeAuthenticationMode = PeerAuthenticationMode.None;
			binding.Port = port;
			binding.MessageAuthentication = false;
			binding.Resolver = new ResolverClient();

			return binding;
		}

		private void Join()
		{
			Bitmap bitmap = new Bitmap(string.Concat(this.student.Name,".bmp"));
			MemoryStream stream = new MemoryStream();
			bitmap.Save(stream,ImageFormat.Bmp);
			stream.Flush();
			byte[] buffer = stream.GetBuffer();

			this.student.Image = buffer;
			this.managementProxy.Join(student);
			this.student.Image = null;
		}

		public void SendResponse(QuizResponse response)
		{
			response.Responder = student;
			this.responseProxy.SendResponse(response);
		}

		private void CloseChannel(IChannel channel)
		{
			if (channel.State == CommunicationState.Opened)
			{
				channel.Close();
			}
		}

		
		#region IQuizQuestion Members

		void IQuizQuestion.AddItem(QuizItem item)
		{
			this.form.AddQuizItem(item);
		}

		#endregion

		#region IQuizResponse Members

		void IQuizResponse.SendResponse(QuizResponse response)
		{
			this.form.DisplayGrade(response);
		}

		#endregion

		
		#region IDisposable Members

		void IDisposable.Dispose()
		{
			if (this.resolverService.State == CommunicationState.Opened)
			{
				this.resolverService.Close();
			}

			if (this.resolverService.State == CommunicationState.Opened)
			{
				this.resolverService.Close();
			}

			this.CloseChannel((IChannel)this.questionProxy);
			this.CloseChannel((IChannel)this.managementProxy);
			this.CloseChannel((IChannel)this.responseProxy);
			
		}

		#endregion

		#region IQuizManagement Members

		void IQuizManagement.Join(Student student)
		{
		}

		void IQuizManagement.Announce(string accouncement)
		{
			this.form.DisplayAnnouncement(accouncement);
		}

		#endregion
}

	
}
