using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.ServiceModel;
using System.Text;
using System.Windows.Forms;

namespace WindowsCommunicationFoundationHandsOn.School
{

	public struct Quiz
	{
		public QuizItem[] Items;
		public QuizResponse[] Responses;
	}


	public partial class MainForm : Form
	{
		private const string ResponseText = "{0} says that the answer to question {1} is {2}";

		private delegate void UpdateStudentsDelegate(byte[] image);
		private delegate void AddResponseDelegate(QuizResponse response);

		private Quiz quiz;
		private QuizTeacher quizService = null;
		private int currentResponse = -1;

		public MainForm()
		{
			InitializeComponent();

		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			this.quizService = new QuizTeacher(this);	

			this.quiz = new Quiz();
			this.quiz.Responses = new QuizResponse[] { };
		
			List<QuizItem> items = new List<QuizItem>();
			
			Bitmap bitmap = null;
			MemoryStream stream = null;
			QuizItem item;
			QuizResponse response;
			List<QuizResponse> responses = null;

			item = new QuizItem();
			item.Identifier = @"1";
			stream = new MemoryStream();
			bitmap = new Bitmap("Allosaurus.jpg");
			bitmap.Save(stream, ImageFormat.Jpeg);
			stream.Flush();
			item.ItemImage = stream.GetBuffer();
			item.Text = @"What is the name of this dinosaur?";

			responses = new List<QuizResponse>();

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "A";
			response.ResponseText = "Allosaurus";
			responses.Add(response);

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "B";
			response.ResponseText = "Brontesaurus";
			responses.Add(response);

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "C";
			response.ResponseText = "T-Rex";
			responses.Add(response);

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "D";
			response.ResponseText = "Velociraptor";
			responses.Add(response);

			item.Responses = responses.ToArray();

			items.Add(item);

			item = new QuizItem();
			item.Identifier = @"2";
			stream = new MemoryStream();
			bitmap = new Bitmap("Triceratops.jpg");
			bitmap.Save(stream, ImageFormat.Jpeg);
			stream.Flush();
			item.ItemImage = stream.GetBuffer();
			item.Text = @"What is the name of this dinosaur?";

			responses = new List<QuizResponse>();

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "A";
			response.ResponseText = "Allosaurus";
			responses.Add(response);

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "B";
			response.ResponseText = "Triceratops";
			responses.Add(response);

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "C";
			response.ResponseText = "T-Rex";
			responses.Add(response);

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "D";
			response.ResponseText = "Velociraptor";
			responses.Add(response);

			item.Responses = responses.ToArray();
			
			items.Add(item);

			item = new QuizItem();
			item.Identifier = @"3";
			stream = new MemoryStream();
			bitmap = new Bitmap("Tyrannosaurus.jpg");
			bitmap.Save(stream, ImageFormat.Jpeg);
			stream.Flush();
			item.ItemImage = stream.GetBuffer();
			item.Text = @"What is the name of this dinosaur?";

			responses = new List<QuizResponse>();

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "A";
			response.ResponseText = "Allosaurus";
			responses.Add(response);

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "B";
			response.ResponseText = "Triceratops";
			responses.Add(response);

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "C";
			response.ResponseText = "T-Rex";
			responses.Add(response);

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "D";
			response.ResponseText = "Velociraptor";
			responses.Add(response);

			item.Responses = responses.ToArray();

			items.Add(item);

			item = new QuizItem();
			item.Identifier = @"4";
			stream = new MemoryStream();
			bitmap = new Bitmap("Diplodocus.jpg");
			bitmap.Save(stream, ImageFormat.Jpeg);
			stream.Flush();
			item.ItemImage = stream.GetBuffer();
			item.Text = @"What is the name of this dinosaur?";

			responses = new List<QuizResponse>();

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "A";
			response.ResponseText = "Allosaurus";
			responses.Add(response);

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "B";
			response.ResponseText = "Triceratops";
			responses.Add(response);

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "C";
			response.ResponseText = "T-Rex";
			responses.Add(response);

			response = new QuizResponse();
			response.Item = item;
			response.ResponseIdentifier = "D";
			response.ResponseText = "Diplodocus";
			responses.Add(response);

			item.Responses = responses.ToArray();

			items.Add(item);

			this.quiz.Items = items.ToArray();


			this.EnableNavigationButtons();
			this.EnableRadioButtons();
			this.EnableAnnouncementButton();
			
		}

		public void UpdateStudents(byte[] image)
		{
			if (this.InvokeRequired)
			{
				UpdateStudentsDelegate updateStudentsDelegate = new UpdateStudentsDelegate(this.UpdateStudents);
				this.Invoke(updateStudentsDelegate, new object[] { image });
			}
			else
			{
				PictureBox student = new PictureBox();
				student.Width = 80;
				student.Height = 50;
				student.SizeMode = PictureBoxSizeMode.StretchImage;
				Padding padding = student.Padding;
				padding.Left = 5; padding.Right = 5;
				padding.Top = 7; padding.Bottom = 7;
				student.Image = new Bitmap(new MemoryStream(image));
				Class.Controls.Add(student);
			}
		}

		public void AddResponse(QuizResponse response)
		{
			if (this.InvokeRequired)
			{
				AddResponseDelegate addResponseDelegate = new AddResponseDelegate(this.AddResponse);
				this.Invoke(addResponseDelegate, new object[] { response });
			}
			else
			{
				lock (this)
				{
					response.Submitted = false;
					List<QuizResponse> responses = new List<QuizResponse>(this.quiz.Responses);
					responses.Add(response);
					this.quiz.Responses = responses.ToArray();

					if (this.quiz.Responses.Length == 1)
					{
						this.currentResponse = 0;
						this.DisplayResponse();
					}

					this.EnableNavigationButtons();
				}
			}
		}

		private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			((IDisposable)this.quizService).Dispose();
		}

		private void StartButton_Click(object sender, EventArgs e)
		{
			this.quizService.SendItems(this.quiz);
			
		}

		private void DisplayResponse()
		{
			QuizResponse response = this.quiz.Responses[this.currentResponse];
			this.QuizResponse.Text = string.Format(MainForm.ResponseText,response.Responder.Name,response.Item.Identifier,response.ResponseText);

			if (!(response.Submitted))
			{
				this.QuizGradeCorrect.Checked = false;
				this.QuizGradeIncorrect.Checked = false;
			}
			else
			{
				this.QuizGradeCorrect.Checked = response.Correct.Value;
				this.QuizGradeIncorrect.Checked = (!(response.Correct.Value));
			}

			this.EnableRadioButtons();
			this.GradeButton.Enabled = false;

		}

		private void SendButton_Click(object sender, EventArgs e)
		{
			this.quizService.SendAccouncement(this.AccouncementBox.Text);
			this.AccouncementBox.Text = string.Empty;
		}

		private void EnableNavigationButtons()
		{
			this.QuizForwardButton.Enabled = (this.currentResponse < this.quiz.Responses.Length - 1);
			this.QuizBackButton.Enabled = (this.currentResponse > 0);
		}

		private void EnableRadioButtons()
		{
			bool enabled = this.currentResponse > -1;
			if (enabled)
			{
				enabled = (!(this.quiz.Responses[this.currentResponse].Submitted));
			}
			this.QuizGradeCorrect.Enabled = enabled;
			this.QuizGradeIncorrect.Enabled = enabled;
		}

		private void QuizBackButton_Click(object sender, EventArgs e)
		{
			this.currentResponse--;
			this.NavigationButton_Click();
		}

		private void QuizForwardButton_Click(object sender, EventArgs e)
		{
			this.currentResponse++;
			this.NavigationButton_Click();
		}

		private void NavigationButton_Click()
		{
			this.EnableNavigationButtons();
			this.DisplayResponse();
			
		}

		private void EnableAnnouncementButton()
		{
			this.SendButton.Enabled = this.AccouncementBox.Text.Length > 0;
		}

		private void QuizGradeCorrect_CheckedChanged(object sender, EventArgs e)
		{
			this.GradeSelected(((RadioButton)sender).Checked);
		}

		private void QuizGradeIncorrect_CheckedChanged(object sender, EventArgs e)
		{
			this.GradeSelected(!(((RadioButton)sender).Checked));
		}

		private void GradeSelected(bool correct)
		{
			this.quiz.Responses[this.currentResponse].Correct = correct;
			this.GradeButton.Enabled = true;
		}

		private void AccouncementBox_TextChanged(object sender, EventArgs e)
		{
			this.EnableAnnouncementButton();
		}

		private void GradeButton_Click(object sender, EventArgs e)
		{
			QuizResponse response = this.quiz.Responses[this.currentResponse];
			if (response.Correct != null)
			{
				this.quizService.SendGrade(response);
			}
			this.GradeButton.Enabled = false;
		}

		
	}
}