using System;
using System.Collections.Generic;
using System.Drawing;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;

namespace WindowsCommunicationFoundationHandsOn.School
{
    [DataContract]
    public struct QuizResponse
    {
        [DataMember]
        public Student Responder;
        [DataMember]
        public QuizItem Item;
        [DataMember]
        public string ResponseText;
        [DataMember]
        public string ResponseIdentifier;
        [DataMember]
        public bool? Correct;
        public bool Submitted;
    }

    [DataContract]
    public struct QuizItem
    {
        [DataMember]
        public string Identifier;
        [DataMember]
        public byte[] ItemImage;
        [DataMember]
        public string Text;
        [DataMember]
        public QuizResponse[] Responses;
        public bool Submitted;

        public string ResponseText
        {
            get
            {
                foreach (QuizResponse response in Responses)
                {
                    if (response.Submitted)
                    {
                        return response.ResponseText;
                    }
                }

                return null;
            }
        }

        public bool? Correct
        {
            get
            {
                foreach (QuizResponse response in Responses)
                {
                    if (response.Submitted)
                    {
                        return response.Correct;
                    }
                }

                return null;
            }
        }
    }

    [DataContract]
    public struct Student
    {
        [DataMember]
        public string Name;
        [DataMember]
        public byte[] Image;
    }





    [ServiceContract(CallbackContract = typeof(IQuizQuestion))]
    [PeerBehavior]
    public interface IQuizQuestion
    {
        [OperationContract(IsOneWay = true)]
        void AddItem(QuizItem item);
    }

    [ServiceContract(CallbackContract = typeof(IQuizResponse))]
    [PeerBehavior]
    public interface IQuizResponse
    {
        [OperationContract(IsOneWay = true)]
        void SendResponse(QuizResponse response);
    }

    [ServiceContract(CallbackContract = typeof(IQuizManagement))]
    [PeerBehavior]
    public interface IQuizManagement
    {
        [OperationContract(IsOneWay = true)]
        void Join(Student student);

        [OperationContract(IsOneWay = true)]
        void Announce(string accouncement);
    }

    public interface IQuizQuestionChannel : IQuizQuestion, IClientChannel
    {
    };

    public interface IQuizResponseChannel : IQuizResponse, IClientChannel
    {
    };

    public interface IQuizManagementChannel : IQuizManagement, IClientChannel
    {
    };


}
