using System;
using System.Collections.Generic;
using System.Configuration;
using System.Drawing;
using System.Security.Cryptography;
using System.Security.Cryptography.Xml;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Security.Tokens;
using System.Text;

namespace WindowsCommunicationFoundationHandsOn.School
{
	public class SecurityValidator : X509CertificateValidator
	{
		public override void Validate(X509Certificate2 certificate)
		{
		}
	}


	public class QuizTeacher : IQuizQuestion, IQuizResponse, IQuizManagement, IDisposable
	{
		private const string BaseAddressKey = "baseAddress";
		private const string QuizPortKey = @"quizPort";
		private const string MeshIdentifierKey = @"MeshIdentifier";
		private const string QuizQuestionEndpointKey = @"quizQuestionEndpoint";
		private const string QuizManagementEndpointKey = @"quizManagementEndpoint";
		private const string QuizResponseEndpointKey = @"quizResponseEndpoint";
		private const string PeerChannelAddressPrefix = @"net.p2p://";
		private const string CurrentUserCertificateStore = @"My";
		private const string CertificateSubjectName = @"Woodgrove";

		private ServiceHost resolverService = null;
		private IQuizQuestionChannel questionProxy = null;
		private IQuizManagementChannel managementProxy = null;
		private int port;
		private string meshIdentifier = null;
		private Resolver resolver = null;
		private InstanceContext site = null;
		private PeerSecurityBehavior peerSecurity;
		
		MainForm form = null;

		Dictionary<string, IQuizResponse> responseProxies = new Dictionary<string, IQuizResponse>();
		
		public QuizTeacher(MainForm form)
		{
			this.form = form;

			Uri baseAddress = new Uri(ConfigurationManager.AppSettings[QuizTeacher.BaseAddressKey]);
			this.resolver = new Resolver();
			this.resolverService = new ServiceHost(resolver, baseAddress);
			this.resolverService.Open();
			
			site = new InstanceContext(this);
			this.port = int.Parse(ConfigurationManager.AppSettings[QuizTeacher.QuizPortKey]);
			this.meshIdentifier = ConfigurationManager.AppSettings[QuizTeacher.MeshIdentifierKey];
			string endpoint = null;
			EndpointAddress address = null;
			
			
			endpoint = ConfigurationManager.AppSettings[QuizTeacher.QuizManagementEndpointKey];
			address = new EndpointAddress(string.Concat(QuizTeacher.PeerChannelAddressPrefix, this.meshIdentifier, "/", endpoint));
			ChannelFactory<IQuizManagementChannel> managementChannelFactory = new ChannelFactory<IQuizManagementChannel>(this.site, this.BuildBinding(this.resolver,this.port), address);
			managementChannelFactory.Open();
			managementProxy = (IQuizManagementChannel)managementChannelFactory.CreateDuplexChannel();
			((IChannel)managementProxy).Open();

			endpoint = ConfigurationManager.AppSettings[QuizTeacher.QuizQuestionEndpointKey];
			address = new EndpointAddress(string.Concat(QuizTeacher.PeerChannelAddressPrefix, this.meshIdentifier, "/", endpoint));
			ChannelFactory<IQuizQuestionChannel> questionChannelFactory = new ChannelFactory<IQuizQuestionChannel>(this.site, this.BuildBinding(this.resolver,this.port), address);
			questionChannelFactory.Open();
			questionProxy = (IQuizQuestionChannel)questionChannelFactory.CreateDuplexChannel();
			((IChannel)questionProxy).Open();

			

			
		}

		private X509Certificate2 GetCertificate()
		{
			X509Store store = new X509Store(QuizTeacher.CurrentUserCertificateStore, StoreLocation.LocalMachine);
			store.Open(OpenFlags.ReadOnly);
			X509CertificateCollection certificateCollection = store.Certificates.Find(X509FindType.FindBySubjectName, QuizTeacher.CertificateSubjectName, false);
			return (X509Certificate2)certificateCollection[0];
		}

		

		private NetPeerTcpBinding BuildBinding(Resolver resolver, int port)
		{
			NetPeerTcpBinding binding = new NetPeerTcpBinding();
			binding.MaxMessageSize = int.MaxValue;
			binding.PeerNodeAuthenticationMode = PeerAuthenticationMode.None;
			binding.MessageAuthentication = false;
			binding.Port = port;
			binding.Resolver = new ResolverClient(resolver);

			return binding;
		}

		public void SendAccouncement(string accouncement)
		{
			managementProxy.Announce(accouncement);
		}

		public void SendItems(Quiz quiz)
		{
			foreach (QuizItem item in quiz.Items)
			{
				questionProxy.AddItem(item);
			}
		}

		public void SendGrade(QuizResponse response)
		{
			if (this.responseProxies.ContainsKey(response.Responder.Name))
			{
				IQuizResponse responseProxy = this.responseProxies[response.Responder.Name];
				responseProxy.SendResponse(response);
			}
			
		}

		private void CloseChannel(IChannel channel)
		{
			if (channel.State == CommunicationState.Opened)
			{
				channel.Close();
			}
		}


		#region IQuizQuestion Members

		void IQuizQuestion.AddItem(QuizItem item)
		{
		}

		#endregion

		#region IQuizResponse Members

		void IQuizResponse.SendResponse(QuizResponse response)
		{
			this.form.AddResponse(response);
		}

		#endregion

		
		#region IDisposable Members

		void IDisposable.Dispose()
		{
			if (this.resolverService.State == CommunicationState.Opened)
			{
				this.resolverService.Close();
			}

			this.CloseChannel((IChannel)this.questionProxy);
			this.CloseChannel((IChannel)this.managementProxy);
			foreach (IQuizResponse responseProxy in this.responseProxies.Values)
			{
				this.CloseChannel((IChannel)responseProxy);
			}
		}

		#endregion

		#region IQuizManagement Members

		void IQuizManagement.Join(Student student)
		{
			if (!(this.responseProxies.ContainsKey(student.Name)))
			{
				string endpoint = ConfigurationManager.AppSettings[QuizTeacher.QuizResponseEndpointKey];
				EndpointAddress address = new EndpointAddress(string.Concat(QuizTeacher.PeerChannelAddressPrefix, this.meshIdentifier, "/", endpoint, "_", student.Name));
				ChannelFactory<IQuizResponseChannel> responseChannelFactory = new ChannelFactory<IQuizResponseChannel>(this.site, this.BuildBinding(this.resolver, this.port), address);
				responseChannelFactory.Open();
				IQuizResponseChannel responseProxy = (IQuizResponseChannel)responseChannelFactory.CreateDuplexChannel();
				((IChannel)responseProxy).Open();
				this.responseProxies.Add(student.Name, responseProxy);

				this.form.UpdateStudents(student.Image);
			}
		}

		void IQuizManagement.Announce(string accouncement)
		{
		}

		#endregion
}

	
}
