﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ServiceModel;
using System.Threading;

namespace WindowsCommunicationFoundationHandsOn.School
{
	public struct Peer
	{
		public Guid identifier;
		public PeerNodeAddress nodeAddress;
		public TimeSpan timeout;
	}

    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
	public class Resolver: IPeerResolver
	{
		private static Dictionary<string, Dictionary<Guid, Peer>> meshes = new Dictionary<string, Dictionary<Guid, Peer>>();

		public object Register(string meshIdentifier, PeerNodeAddress nodeAddress, TimeSpan timeout)
		{
			lock (this)
			{
				Dictionary<Guid, Peer> mesh = null;
				if (!(meshes.ContainsKey(meshIdentifier)))
				{
					mesh = new Dictionary<Guid, Peer>();
					meshes.Add(meshIdentifier, mesh);
				}
				else
				{
					mesh = meshes[meshIdentifier];
				}

				Peer peer = new Peer();
				peer.identifier = Guid.NewGuid();
				peer.nodeAddress = nodeAddress;
				peer.timeout = timeout;
				mesh.Add(peer.identifier, peer);

				return peer.identifier;
			}
		}

		public PeerNodeAddress[] Resolve(string meshIdentifier, int maximumAddresses, TimeSpan timeout)
		{
			List<PeerNodeAddress> addresses = new List<PeerNodeAddress>();
			if (meshes.ContainsKey(meshIdentifier))
			{
				Dictionary<Guid, Peer> mesh = meshes[meshIdentifier];
				foreach (Peer peer in mesh.Values)
				{
					addresses.Add(peer.nodeAddress);
					if (addresses.Count >= maximumAddresses)
					{
						break;
					}
				}
			}
			return addresses.ToArray();
			
		}

		public void Unregister(object registrationId, TimeSpan timeout)
		{
			lock (this)
			{
				foreach (Dictionary<Guid, Peer> mesh in meshes.Values)
				{
					if (mesh.ContainsKey((Guid)registrationId))
					{
						mesh.Remove((Guid)registrationId);
					}
				}
			}
		}

		public void Update(object registrationId, PeerNodeAddress updatedNodeAddress, TimeSpan timeout)
		{
			lock (this)
			{
				Peer peer;
				foreach (Dictionary<Guid, Peer> mesh in meshes.Values)
				{
					if (mesh.ContainsKey((Guid)registrationId))
					{
						peer = mesh[(Guid)registrationId];
						peer.nodeAddress = updatedNodeAddress;
					}
				}
			}
		}
	}
}
