using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.Text;
using System.Transactions;

namespace Fabrikam
{
    internal class DealData: IExtension<InstanceContext>
    {
        private string dealIdentifier = null;
        private Trade[] trades = null;

        public DealData(string dealIdentifier)
        {
            this.dealIdentifier = dealIdentifier;
            this.trades = new Trade[]{};
        }

        public string DealIdentifier
        {
            get
            {
                return this.dealIdentifier;
            }
        }

        public void AddTrade(Trade trade)
        {
            List<Trade> tradeList = null;
            tradeList = new List<Trade>(this.trades);
            tradeList.Add(trade);
            this.trades = tradeList.ToArray();
            
        }

        public Trade[] Trades
        {
            get
            {
                return this.trades;
            }
        }

        #region IExtension<InstanceContext> Members

        void IExtension<InstanceContext>.Attach(InstanceContext owner)
        {
            
        }

        void IExtension<InstanceContext>.Detach(InstanceContext owner)
        {
            
        }

        #endregion
    }

    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerSession)]
    public class TradingSystem: ITradingService
    {
        #region ITradingService Members

        string ITradingService.BeginDeal()
        {
            string dealIdentifier = Guid.NewGuid().ToString();

            OperationContext.Current.InstanceContext.Extensions.Add(new DealData(dealIdentifier));
            
            Console.WriteLine(string.Format("Started deal {0}",dealIdentifier));
            return dealIdentifier;
        }

        void ITradingService.AddTrade(Trade trade)
        {
            DealData dealData = OperationContext.Current.InstanceContext.Extensions.Find<DealData>();
            dealData.AddTrade(trade);
            Console.WriteLine(string.Format("Added trade for {0}",trade));
        }

        void ITradingService.AddFunction(string function)
        {
            Console.WriteLine(string.Format("Added function {0}",function));
        }

        decimal ITradingService.Calculate()
        {
            Decimal value = DateTime.Now.Millisecond/10;
            Console.WriteLine(string.Format("Calculated value as {0}",value));
            return value;
        }

        [OperationBehavior(TransactionScopeRequired=true,TransactionAutoComplete=true)]
        void ITradingService.Purchase()
        {
            DealData dealData = OperationContext.Current.InstanceContext.Extensions.Find<DealData>();
            ITradeRecorder proxy = new ChannelFactory<ITradeRecorder>("TradeRecordingService").CreateChannel();
            proxy.RecordTrades(dealData.Trades);
            Console.WriteLine("Purchased!");
        }

        void ITradingService.EndDeal()
        {
            DealData dealData = OperationContext.Current.InstanceContext.Extensions.Find<DealData>();
            Console.WriteLine(string.Format("Completed deal: {0}",dealData.DealIdentifier));
        }

        #endregion
    }
}
