// ----------------------------------------------------------------------------
// Copyright (C) 2003-2005 Microsoft Corporation, All rights reserved.
// ----------------------------------------------------------------------------

namespace Microsoft.ServiceModel.Samples
{
    #region using
    using System;
    using System.Configuration;
    using System.ServiceModel;
    using System.ServiceModel.Channels;
    using System.ServiceModel.Configuration;
    #endregion

    /// <summary>
    /// Configuration section for Udp. 
    /// </summary>
    public class MailTransportSection : BindingElementExtensionSection
    {
        public MailTransportSection()
        {
        }

        #region Configuration_Properties
        [ConfigurationProperty(MailConfigurationStrings.MaxBufferPoolSize, DefaultValue = MailDefaults.MaxBufferPoolSize)]
        [LongValidator(MinValue = 0)]
        public long MaxBufferPoolSize
        {
            get { return (long)base[MailConfigurationStrings.MaxBufferPoolSize]; }
            set { base[MailConfigurationStrings.MaxBufferPoolSize] = value; }
        }

        [ConfigurationProperty(MailConfigurationStrings.MaxMessageSize, DefaultValue = MailDefaults.MaxMessageSize)]
        [IntegerValidator(MinValue = 1)]
        public int MaxMessageSize
        {
            get { return (int)base[MailConfigurationStrings.MaxMessageSize]; }
            set { base[MailConfigurationStrings.MaxMessageSize] = value; }
        }

        [ConfigurationProperty(MailConfigurationStrings.Multicast, DefaultValue = MailDefaults.Multicast)]
        public bool Multicast
        {
            get { return (bool)base[MailConfigurationStrings.Multicast]; }
            set { base[MailConfigurationStrings.Multicast] = value; }
        }
        #endregion

        public override Type BindingElementType
        {
            get { return typeof(MailTransportBindingElement); }
        }

        public override string ConfiguredSectionName
        {
            get { return MailConstants.UdpTransportSectionName; }
        }

        protected override BindingElement CreateBindingElement()
        {
            MailTransportBindingElement bindingElement = new MailTransportBindingElement();
            this.ApplyConfiguration(bindingElement);
            return bindingElement;
        }

        #region Configuration_Infrastructure_overrides
        public override void ApplyConfiguration(BindingElement bindingElement)
        {
            base.ApplyConfiguration(bindingElement);

            MailTransportBindingElement udpBindingElement = (MailTransportBindingElement)bindingElement;
            udpBindingElement.MaxBufferPoolSize = this.MaxBufferPoolSize;
            udpBindingElement.MaxMessageSize = this.MaxMessageSize;
            udpBindingElement.Multicast = this.Multicast;
        }

        public override void CopyFrom(ServiceModelExtensionSection from)
        {
            base.CopyFrom(from);

            MailTransportSection source = (MailTransportSection)from;
            this.MaxBufferPoolSize = source.MaxBufferPoolSize;
            this.MaxMessageSize = source.MaxMessageSize;
            this.Multicast = source.Multicast;
        }

        protected override void InitializeFrom(BindingElement bindingElement)
        {
            base.InitializeFrom(bindingElement);

            MailTransportBindingElement udpBindingElement = (MailTransportBindingElement)bindingElement;
            this.MaxBufferPoolSize = udpBindingElement.MaxBufferPoolSize;
            this.MaxMessageSize = udpBindingElement.MaxMessageSize;
            this.Multicast = udpBindingElement.Multicast;
        }

        protected override ConfigurationPropertyCollection Properties
        {
            get
            {
                ConfigurationPropertyCollection properties = base.Properties;
                properties.Add(new ConfigurationProperty(MailConfigurationStrings.MaxBufferPoolSize,
                    typeof(long), MailDefaults.MaxBufferPoolSize, null, new LongValidator(0, Int64.MaxValue), ConfigurationPropertyOptions.None));
                properties.Add(new ConfigurationProperty(MailConfigurationStrings.MaxMessageSize,
                    typeof(int), MailDefaults.MaxMessageSize, null, new IntegerValidator(1, Int32.MaxValue), ConfigurationPropertyOptions.None));
                properties.Add(new ConfigurationProperty(MailConfigurationStrings.Multicast,
                    typeof(Boolean), MailDefaults.Multicast, null, null, ConfigurationPropertyOptions.None));
                return properties;
            }
        }
        #endregion
    }
}
