// ----------------------------------------------------------------------------
// Copyright (C) 2003-2005 Microsoft Corporation, All rights reserved.
// ----------------------------------------------------------------------------

namespace Microsoft.ServiceModel.Samples
{
    using System;
    using System.Configuration;
    using System.Collections;
    using System.Globalization;
    using System.ServiceModel;
    using System.ServiceModel.Configuration;

    public class SampleProfileMailBindingConfigurationElement : StandardBindingConfigurationElement
    {
        public SampleProfileMailBindingConfigurationElement(string configurationName)
            : base(configurationName)
        {
        }

        public SampleProfileMailBindingConfigurationElement()
            : this(null)
        {
        }

        protected override Type BindingElementType
        {
            get { return typeof(SampleProfileMailBinding); }
        }

        [ConfigurationProperty(MailConfigurationStrings.OrderedSession, DefaultValue = MailDefaults.OrderedSession)]
        public bool OrderedSession
        {
            get { return (bool)base[MailConfigurationStrings.OrderedSession]; }
            set { base[MailConfigurationStrings.OrderedSession] = value; }
        }

        [ConfigurationProperty(MailConfigurationStrings.ReliableSessionEnabled, DefaultValue = MailDefaults.ReliableSessionEnabled)]
        public bool ReliableSessionEnabled
        {
            get { return (bool)base[MailConfigurationStrings.ReliableSessionEnabled]; }
            set { base[MailConfigurationStrings.ReliableSessionEnabled] = value; }
        }

        [ConfigurationProperty(MailConfigurationStrings.SessionInactivityTimeout, DefaultValue = MailDefaults.SessionInactivityTimeoutString)]
        [TimeSpanValidator(MinValueString = "00:00:00")]
        public TimeSpan SessionInactivityTimeout
        {
            get { return (TimeSpan)base[MailConfigurationStrings.SessionInactivityTimeout]; }
            set { base[MailConfigurationStrings.SessionInactivityTimeout] = value; }
        }

        protected override ConfigurationPropertyCollection Properties
        {
            get
            {
                ConfigurationPropertyCollection properties = base.Properties;
                properties.Add(new ConfigurationProperty(MailConfigurationStrings.OrderedSession, 
                    typeof(Boolean), MailDefaults.OrderedSession, null, null, ConfigurationPropertyOptions.None));
                properties.Add(new ConfigurationProperty(MailConfigurationStrings.ReliableSessionEnabled, 
                    typeof(Boolean), MailDefaults.ReliableSessionEnabled, null, null, ConfigurationPropertyOptions.None));
                properties.Add(new ConfigurationProperty(MailConfigurationStrings.SessionInactivityTimeout, 
                    typeof(TimeSpan), TimeSpan.Parse(MailDefaults.SessionInactivityTimeoutString), null, 
                    new TimeSpanValidator(TimeSpan.Parse("00:00:00"), TimeSpan.Parse("10675199.02:48:05.4775807"), false), 
                    ConfigurationPropertyOptions.None));
                return properties;
            }
        }        
        
        protected override void InitializeFrom(Binding binding)
        {
            base.InitializeFrom(binding);
            SampleProfileMailBinding udpBinding = (SampleProfileMailBinding)binding;

            this.OrderedSession = udpBinding.OrderedSession;
            this.ReliableSessionEnabled = udpBinding.ReliableSessionEnabled;
            this.SessionInactivityTimeout = udpBinding.SessionInactivityTimeout;
        }

        protected override void OnApplyConfiguration(Binding binding)
        {
            if (binding == null)
                throw new ArgumentNullException("binding");

            if (binding.GetType() != typeof(SampleProfileMailBinding))
            {
                throw new ArgumentException(string.Format(CultureInfo.CurrentCulture,
                    "Invalid type for binding. Expected type: {0}. Type passed in: {1}.",
                    typeof(SampleProfileMailBinding).AssemblyQualifiedName,
                    binding.GetType().AssemblyQualifiedName));
            }
            SampleProfileMailBinding udpBinding = (SampleProfileMailBinding)binding;

            udpBinding.OrderedSession = this.OrderedSession;
            udpBinding.ReliableSessionEnabled = this.ReliableSessionEnabled;
            udpBinding.SessionInactivityTimeout = this.SessionInactivityTimeout;
        }
    }
}
