using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.Text;
using System.Transactions;

namespace Client
{
    public class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("Press any key when the service is ready.");
            Console.ReadKey();

            
            try
            {
                //do
                for(int i = 0;i < 50;i++)
                {
                    Console.WriteLine();

                    

                    using(TransactionScope scope = new TransactionScope(TransactionScopeOption.RequiresNew))
                    {
                        decimal? dealValue = null;
                        decimal? hedgeValue = null;

                        ITradingService dealProxy = null;
                        ITradingService hedgeProxy = null;
                        try
                        {
                            dealProxy = new ChannelFactory<ITradingService>("TradingService").CreateChannel();

                            dealProxy.BeginDeal();

                            Trade trade = new Trade();
                            trade.Count = 10;
                            trade.Symbol = "MSFT";
                            trade.Date = DateTime.Now.AddMonths(2);
                            dealProxy.AddTrade(trade);

                            dealProxy.AddFunction("InterestRateEstimation");
                            dealProxy.AddFunction("TechnologyStockEstimation");

                            dealValue = dealProxy.Calculate();

                            dealProxy.Purchase();

                            dealProxy.EndDeal();  
                      
                            
                       
                            hedgeProxy = new ChannelFactory<ITradingService>("TradingService").CreateChannel();

                            hedgeProxy.BeginDeal();

                            trade = new Trade();
                            trade.Count = 10;
                            trade.Symbol = "MSFT";
                            trade.Date = DateTime.Now.AddMonths(2);
                            hedgeProxy.AddTrade(trade);

                            hedgeProxy.AddFunction("InterestRateEstimation");
                            hedgeProxy.AddFunction("TechnologyStockEstimation");

                            hedgeValue = hedgeProxy.Calculate();

                            hedgeProxy.Purchase();

                            hedgeProxy.EndDeal();  
                      
                             if((dealValue != null)&&(hedgeValue != null))
                             {
                                Console.WriteLine("Deal value is ${0}, and hedge value is ${1}",dealValue.Value,hedgeValue.Value);
                                if((dealValue.Value - hedgeValue.Value) > 20m)
                                {
                                    Console.WriteLine("Voting to complete trade!");
                                    scope.Complete();
                                }
                             }
                        }
                        catch(Exception exception)
                        {
                            Console.WriteLine(exception.Message);
                        }
                        finally
                        {
                            ((IChannel)dealProxy).Close();
                            ((IChannel)hedgeProxy).Close();
                            
                        }
                    }

                    //Console.WriteLine("Press q to quit or any other key to continue.");

                }
                //while(Console.ReadKey().KeyChar != 'q');
            }
            catch(Exception exception)
            {
                Console.WriteLine(exception.Message);
            }
           
            
            Console.WriteLine();
            Console.WriteLine("Done.");
            Console.ReadKey();
        }
    }
}
