using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.Text;

namespace WindowsCommunicationFoundationHandsOn.School
{
	public class ResolverClient : PeerResolver
	{
		private const string PeerResolverServiceConfiguration = "PeerResolverService";

		public override object Register(string meshId, PeerNodeAddress nodeAddress, TimeSpan timeout)
		{
			IPeerResolver proxy = null;
			try
			{
				proxy = new ChannelFactory<IPeerResolver>(ResolverClient.PeerResolverServiceConfiguration).CreateChannel();
				object registrationId = proxy.Register(meshId, nodeAddress, timeout);
				return registrationId;
			}
			catch (Exception)
			{
				return null;
			}
			finally
			{
				IChannel channel = (IChannel)proxy;
				if (channel.State == CommunicationState.Opened)
				{
					channel.Close();
				}
			}
		}

		public override System.Collections.ObjectModel.ReadOnlyCollection<PeerNodeAddress> Resolve(string meshId, int maxAddresses, TimeSpan timeout)
		{
			IPeerResolver proxy = null;
			try
			{
				System.Collections.ObjectModel.ReadOnlyCollection<PeerNodeAddress> addresses = null;
				proxy = new ChannelFactory<IPeerResolver>(ResolverClient.PeerResolverServiceConfiguration).CreateChannel();
				PeerNodeAddress[] addressArray = proxy.Resolve(meshId, maxAddresses, timeout);
				addresses = new System.Collections.ObjectModel.ReadOnlyCollection<PeerNodeAddress>(addressArray);
				return addresses;
			}
			catch (Exception)
			{
				return null;
			}
			finally
			{
				IChannel channel = (IChannel)proxy;
				if (channel.State == CommunicationState.Opened)
				{
					channel.Close();
				}
			}
		}

		public override void Unregister(object registrationId, TimeSpan timeout)
		{
			IPeerResolver proxy = null;
			try
			{
				proxy = new ChannelFactory<IPeerResolver>(ResolverClient.PeerResolverServiceConfiguration).CreateChannel();
				proxy.Unregister(registrationId, timeout);
			}
			catch (Exception)
			{
			}
			finally
			{
				IChannel channel = (IChannel)proxy;
				if (channel.State == CommunicationState.Opened)
				{
					channel.Close();
				}
			}
		}

		public override void Update(object registrationId, PeerNodeAddress updatedNodeAddress, TimeSpan timeout)
		{
			IPeerResolver proxy = null;
			try
			{
				proxy = new ChannelFactory<IPeerResolver>(ResolverClient.PeerResolverServiceConfiguration).CreateChannel();
				proxy.Update(registrationId, updatedNodeAddress, timeout);
			}
			catch (Exception)
			{
			}
			finally
			{
				IChannel channel = (IChannel)proxy;
				if (channel.State == CommunicationState.Opened)
				{
					channel.Close();
				}
			
			}
		}
	}
}

