using System;
using System.ServiceModel;
using System.ServiceModel.Security.Protocols;
using System.Xml;

namespace SecurityTokenService
{
    public abstract class SecurityTokenServiceBase : ISecurityTokenService
    {
        bool log;

        protected SecurityTokenServiceBase()
            : this(false)
        {
        }

        protected SecurityTokenServiceBase(bool log)
        {
            this.log = log;
        }

        public abstract RequestSecurityTokenResponse Issue(RequestSecurityToken request);

        public virtual RequestSecurityTokenResponse IssueChallenge(RequestSecurityTokenResponse challenge)
        {
            throw new NotImplementedException();
        }

        public virtual RequestSecurityTokenResponse Renew(RequestSecurityToken request)
        {
            throw new NotImplementedException();
        }

        public virtual RequestSecurityTokenResponse RenewChallenge(RequestSecurityTokenResponse challenge)
        {
            throw new NotImplementedException();
        }

        public virtual RequestSecurityTokenResponse Cancel(RequestSecurityToken request)
        {
            throw new NotImplementedException();
        }

        public virtual RequestSecurityTokenResponse Validate(RequestSecurityToken request)
        {
            throw new NotImplementedException();
        }

        Message ISecurityTokenService.Issue(Message requestMessage)
        {
            if (log)
            {
                Console.WriteLine("---REQUEST MESSAGE---------------------------------------");
                Console.WriteLine(requestMessage);
            }

            RequestSecurityToken rst = RequestSecurityToken.CreateFrom(requestMessage.GetReaderAtBodyContents());

            if (rst.RequestType != null && rst.RequestType != "http://schemas.xmlsoap.org/ws/2005/02/trust/Issue")
                throw new InvalidOperationException(rst.RequestType);

            Message responseMessage = Message.CreateMessage("http://schemas.xmlsoap.org/ws/2005/02/trust/RSTR/Issue", Issue(rst));

            if (log)
            {
                Console.WriteLine("---RESPONSE MESSAGE--------------------------------------");
                Console.WriteLine(responseMessage);
            }

            return responseMessage;
        }

        Message ISecurityTokenService.IssueChallenge(Message requestMessage)
        {
            if (log)
            {
                Console.WriteLine("---REQUEST MESSAGE---------------------------------------");
                Console.WriteLine(requestMessage);
            }

            RequestSecurityTokenResponse rstr = RequestSecurityTokenResponse.CreateFrom(requestMessage.GetReaderAtBodyContents());
            Message responseMessage = Message.CreateMessage("http://schemas.xmlsoap.org/ws/2005/02/trust/RSTR/Issue", IssueChallenge(rstr));

            if (log)
            {
                Console.WriteLine("---RESPONSE MESSAGE--------------------------------------");
                Console.WriteLine(responseMessage);
            }

            return responseMessage;
        }

        Message ISecurityTokenService.Renew(Message requestMessage)
        {
            if (log)
            {
                Console.WriteLine("---REQUEST MESSAGE---------------------------------------");
                Console.WriteLine(requestMessage);
            }

            RequestSecurityToken rst = RequestSecurityToken.CreateFrom(requestMessage.GetReaderAtBodyContents());

            if (rst.RequestType != null && rst.RequestType != "http://schemas.xmlsoap.org/ws/2005/02/trust/Renew")
                throw new InvalidOperationException(rst.RequestType);

            Message responseMessage = Message.CreateMessage("http://schemas.xmlsoap.org/ws/2005/02/trust/RSTR/Renew", Renew(rst));

            if (log)
            {
                Console.WriteLine("---RESPONSE MESSAGE--------------------------------------");
                Console.WriteLine(responseMessage);
            }

            return responseMessage;
        }

        Message ISecurityTokenService.RenewChallenge(Message requestMessage)
        {
            if (log)
            {
                Console.WriteLine("---REQUEST MESSAGE---------------------------------------");
                Console.WriteLine(requestMessage);
            }

            RequestSecurityTokenResponse rstr = RequestSecurityTokenResponse.CreateFrom(requestMessage.GetReaderAtBodyContents());
            Message responseMessage = Message.CreateMessage("http://schemas.xmlsoap.org/ws/2005/02/trust/RSTR/Issue", RenewChallenge(rstr));

            if (log)
            {
                Console.WriteLine("---RESPONSE MESSAGE--------------------------------------");
                Console.WriteLine(responseMessage);
            }

            return responseMessage;
        }

        Message ISecurityTokenService.Cancel(Message requestMessage)
        {
            if (log)
            {
                Console.WriteLine("---REQUEST MESSAGE---------------------------------------");
                Console.WriteLine(requestMessage);
            }

            RequestSecurityToken rst = RequestSecurityToken.CreateFrom(requestMessage.GetReaderAtBodyContents());

            if (rst.RequestType != null && rst.RequestType != "http://schemas.xmlsoap.org/ws/2005/02/trust/Cancel")
                throw new InvalidOperationException(rst.RequestType);

            Message responseMessage = Message.CreateMessage("http://schemas.xmlsoap.org/ws/2005/02/trust/RSTR/Cancel", Cancel(rst));

            if (log)
            {
                Console.WriteLine("---RESPONSE MESSAGE--------------------------------------");
                Console.WriteLine(responseMessage);
            }

            return responseMessage;
        }

        Message ISecurityTokenService.Validate(Message requestMessage)
        {
            if (log)
            {
                Console.WriteLine("---REQUEST MESSAGE---------------------------------------");
                Console.WriteLine(requestMessage);
            }

            RequestSecurityToken rst = RequestSecurityToken.CreateFrom(requestMessage.GetReaderAtBodyContents());

            if (rst.RequestType != null && rst.RequestType != "http://schemas.xmlsoap.org/ws/2005/02/trust/Validate")
                throw new InvalidOperationException(rst.RequestType);

            Message responseMessage = Message.CreateMessage("http://schemas.xmlsoap.org/ws/2005/02/trust/RSTR/Validate", Validate(rst));

            if (log)
            {
                Console.WriteLine("---RESPONSE MESSAGE--------------------------------------");
                Console.WriteLine(responseMessage);
            }

            return responseMessage;
        }
    }
}
