using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Security.Authorization;
using System.ServiceModel;
using System.Web;
using System.Web.Security;

namespace Service
{

    public class AccessChecker : OperationRequirement
    {
        private Dictionary<string, Claim[]> accessRequirements = null;

        public AccessChecker()
        {
            this.accessRequirements = new Dictionary<string, Claim[]>();

            OperationRequirementsConfigurationSection
                operationRequirementsConfigurationSection
                = ConfigurationManager.GetSection("operationRequirements")
                as OperationRequirementsConfigurationSection;

            OperationRequirementsCollection requirements =
                operationRequirementsConfigurationSection.OperationRequirements;
            List<Claim> roleClaims = null;
            foreach (OperationElement operationElement in requirements)
            {
                roleClaims = new List<Claim>(operationElement.Roles.Count);
                foreach (RoleElement roleElement in operationElement.Roles)
                {
                    roleClaims.Add(
                        new Claim(
                "http://schemas.microsoft.com/xsi/2005/05/ClaimType/:Role",
                            roleElement.Name, 
                            Rights.PossessProperty));
                }

                this.accessRequirements.Add(
                    operationElement.Identifier,
                    roleClaims.ToArray());
            }
            
        }

        public override bool AccessCheck(OperationContext operationContext)
        {
            string header =
                operationContext.RequestContext.RequestMessage.Headers.Action;
            Claim[] requiredClaims = null;
            if (!(accessRequirements.TryGetValue(header, out requiredClaims)))
            {
                return false;
            }

            AuthorizationContext authorizationContext =
                operationContext.ServiceSecurityContext.AuthorizationContext;

            foreach (Claim requiredClaim in requiredClaims)
            {
                for (
                    int index = 0;
                    index < authorizationContext.ClaimSets.Count;
                    index++)
                {
                    if (
                        authorizationContext.ClaimSets[index].
                        ContainsClaim(requiredClaim))
                    {
                        return true;
                    }
                }
            }
            return false;
        }
    }
}